
% import data
sample_data = importdata('sample_data.csv'); % data is (N x T) x K0 matrix 

T = 16;    % time length

N = 507;  % number of cross-sectional units

K0 = 4;    % number of imported variables including a dependent variable
K = K0-1; % number of exogenous variables


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Since `data' is (N x T) x K0 matrix,  it needs to be converted such that T x N x K0 


% Y_NT = reshape(sample_data(:,1), T, N );     % T x N 
% X_NT = [];
% for k=1:K
%     X_NT = cat(3, X_NT, reshape(sample_data(:,k+1), T, N ));  % T x N x K
% end

DATA = [];
for k=1:K0
    DATA = cat(3, DATA, reshape(sample_data(:,k), T, N ));  % T x N x K
end


% DATA = cat(3,Y_NT,X_NT); % DATA must be T x N x K0 with the first sheet being dependent variable y.

%%%% Estimate by transformed ML estimator %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [coef, var_nonrobust, var_robust, sigma2] = TransML(DATA,time_effect,ini_MDE)
% INPUT: 
% DATA => T x N x (K+1) where K is the number of exogenous variables
% 
% ini_MDE => option for the starting values that are used for numerical optimization
%     ini_MDE = 0: random variable 
%     ini_MDE = 1: Minimum distance estimator developed by Hsiao, Pesaran and Tahmiscioglu (2002,JoE)  
% 
% time_effect => option for including time effects
%     time_effect = 0: time effects are not included in the estimation 
%     time_effect = 1: time effects are included in the estimation  
% 
% OUTPUT:
% coef         : coefficient of gamma and beta's 
% var_nonrobust: nonrobust covariance matrix of coef
% var_robust   : robust covariance matrix of coef
% sigma2       : estimate of sigma2 = var(u_it)
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ini_MDE = 0;
time_effect = 0;
[ coef_ML1, var_ML_nonrobust1, var_ML_robust1, sigma_ML1 ] = TransML( DATA, time_effect, ini_MDE );
 se_ML_nonrobust1 = sqrt( diag( var_ML_nonrobust1 ) );
 se_ML_robust1   = sqrt( diag( var_ML_robust1 ) );


ini_MDE = 1;
time_effect = 0;
 [ coef_ML2, var_ML_nonrobust2, var_ML_robust2, sigma_ML2 ] = TransML( DATA, time_effect, ini_MDE );
 se_ML_nonrobust2 = sqrt( diag( var_ML_nonrobust2 ) );
 se_ML_robust2   = sqrt( diag( var_ML_robust2 ) );


ini_MDE = 0;
time_effect = 1;
 [ coef_ML3, var_ML_nonrobust3, var_ML_robust3, sigma_ML3 ] = TransML( DATA, time_effect, ini_MDE );
 se_ML_nonrobust3 = sqrt( diag( var_ML_nonrobust3 ) );
 se_ML_robust3   = sqrt( diag( var_ML_robust3 ) );

ini_MDE = 1;
time_effect = 1;
 [ coef_ML4, var_ML_nonrobust4, var_ML_robust4, sigma_ML4 ] = TransML( DATA, time_effect, ini_MDE );
 se_ML_nonrobust4 = sqrt( diag( var_ML_nonrobust4 ) );
 se_ML_robust4   = sqrt( diag( var_ML_robust4 ) );


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

format = '  %1.4f    ';
name = [     '    y(-1)    ';    '     x1      ';     '     x2      ';     '     x3      ';       ];

output1 = [coef_ML1  se_ML_nonrobust1   se_ML_robust1     ];  
output2 = [coef_ML2  se_ML_nonrobust2   se_ML_robust2     ];  
output3 = [coef_ML3  se_ML_nonrobust3   se_ML_robust3     ];  
output4 = [coef_ML4  se_ML_nonrobust4   se_ML_robust4     ];  

output1 = num2str(output1, format);
output2 = num2str(output2, format);
output3 = num2str(output3, format);
output4 = num2str(output4, format);

disp('  '); 
disp('**************************************************  ');
disp('   starting value : random variable         '); 
disp('   time effects   : not included '); 
disp('  '); 
disp('                       non robust     robust                ');
disp('  variable    coef      std. err     std. err                 ');
disp([ name  output1]);
disp('  '); 
disp('**************************************************  ');
disp('   starting value : minimum distance estimator          '); 
disp('   time effects   : not included '); 
disp('  '); 
disp('                       non robust     robust                ');
disp('  variable    coef      std. err     std. err                 ');
disp([ name  output2]);
disp('  '); 
disp('**************************************************  ');
disp('   starting value : random variable         '); 
disp('   time effects   : included '); 
disp('  '); 
disp('                       non robust     robust                ');
disp('  variable    coef      std. err     std. err                 ');
disp([ name  output3]);
disp('  '); 
disp('**************************************************  ');
disp('   starting value : minimum distance estimator          '); 
disp('   time effects   : included '); 
disp('  '); 
disp('                       non robust     robust                ');
disp('  variable    coef      std. err     std. err                 ');
disp([ name  output4]);
disp('  '); 










